#include <curl/curl.h>

#include <sstream>
#include <string>
#include "histogram.h"
#include "text.h"
#include "svg.h"

struct Input {
    vector<double> vec;
    size_t korz{};
};
Input input_data(istream& in, bool prompt = false) {
    Input lin;
    size_t n, korz;
    if(prompt)
        cerr << "Number of elements: ";
    in >> n;
    lin.vec.resize(n);
    for (size_t i = 0; i < n; i++)
        in >> lin.vec[i];
    if(prompt)
        cerr << "Enter bin count: ";
    in >> lin.korz;
    return lin;
}

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx){
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    size_t data_size = item_size * item_count;
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input download(const string& address){
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if (curl) {
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        CURLcode res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if (res != CURLE_OK) {
            cerr << "cURL error: " << curl_easy_strerror(res) << endl;
            exit(1);
        }
    return input_data(buffer, false);
}
}

int main(int argc, char* argv[]) {
    curl_global_init(CURL_GLOBAL_ALL);
    Input input;
    if (argc > 1) {
       input = download(argv[1]);
    }else{
        input = input_data(cin, true);
    }
    auto bins = make_histogram(input.korz, input.vec);
    show_histogram_svg(bins);
    return 0;
}

//  .   ,    ( )    .
//       ,  , ,      ,  .
//    ,         .
// NUL           .

/*
C:\Users\Natasha\Desktop\ 2\cs-lab34\bin\Debug>cs-lab34.exe -x --y /z w
argc = 5
argv[0] = cs-lab34.exe
argv[1] = -x
argv[2] = --y
argv[3] = /z
argv[4] = w

C:\Users\Natasha\Desktop\ 2\cs-lab34\bin\Debug>cs-lab34.exe param "with spaces"
argc = 3
argv[0] = cs-lab34.exe
argv[1] = param
argv[2] = with spaces

C:\Users\Natasha\Desktop\ 2\cs-lab34\bin\Debug>cs-lab34.exe param <marks.txt >NUL
*/
